@@ -11,8 +11,9 @@ import conf_configParser as confmodule
 
 """Module for using gconf without having to care about types"""
 
-def notify_add (key, func):
-    return confmodule.notify_add(key, func)
+def notify_add (key, func, *args):
+    """The signature for func must be self, client, *args, **kwargs"""
+    return confmodule.notify_add(key, func, args)
 
 def notify_remove (conid):
     confmodule.notify_remove(conid)
@@ -33,3 +34,19 @@ def set (key, value):
 
 def hasKey (key):
     return confmodule.hasKey(key)
+
+import sys, os
+if sys.platform == "win32":
+    username = os.environ["USERNAME"]
+    del sys, os
+else:
+    from os import getuid
+    from pwd import getpwuid
+    userdata = getpwuid(getuid())
+    realname = userdata.pw_gecos.split(",")[0]
+    if realname:
+        username = realname
+    else: username = userdata.pw_name
+    del getuid, getpwuid
+    del sys, os
+    del userdata, realname
