@@ -1,22 +1,50 @@
-
+from collections import defaultdict
 from threading import RLock
 import traceback
 import cStringIO
 import datetime
+import Queue
 
 from gobject import SIGNAL_RUN_FIRST, TYPE_NONE, GObject
 
 from pychess.Savers.ChessFile import LoadingError
 from pychess.Players.Player import PlayerIsDead, TurnInterrupt
-from pychess.System.ThreadPool import pool, PooledThread
+from pychess.System.ThreadPool import PooledThread, pool
 from pychess.System.protoopen import protoopen, protosave, isWriteable
 from pychess.System.Log import log
 from pychess.System import glock
+from pychess.Variants.normal import NormalChess
 
-from Board import Board
-from logic import getStatus, isClaimableDraw
+from logic import getStatus, isClaimableDraw, playerHasMatingMaterial
 from const import *
 
+def undolocked (f):
+    def newFunction(*args, **kw):
+        self = args[0]
+        log.debug("undolocked: adding func to queue: %s %s %s\n" % \
+            (repr(f), repr(args), repr(kw)))
+        self.undoQueue.put((f, args, kw))
+        
+        locked = self.undoLock.acquire(blocking=False)        
+        if locked:
+            try:
+                while True:
+                    try:
+                        func, args, kw = self.undoQueue.get_nowait()
+                        log.debug("undolocked: running queued func: %s %s %s\n" % \
+                            (repr(func), repr(args), repr(kw)))
+                        func(*args, **kw)
+                    except Queue.Empty:
+                        break
+            finally:
+                self.undoLock.release()
+    return newFunction
+
+def inthread (f):
+    def newFunction(*args, **kw):
+        pool.start(f, *args, **kw)
+    return newFunction
+
 class GameModel (GObject, PooledThread):
     
     """ GameModel contains all available data on a chessgame.
@@ -26,17 +54,22 @@ class GameModel (GObject, PooledThread):
         "game_started":  (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
         "game_changed":  (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
         "moves_undoing": (SIGNAL_RUN_FIRST, TYPE_NONE, (int,)),
-        "game_loading":  (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
+        "game_unended":  (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
+        "game_loading":  (SIGNAL_RUN_FIRST, TYPE_NONE, (object,)),
         "game_loaded":   (SIGNAL_RUN_FIRST, TYPE_NONE, (object,)),
         "game_saved":    (SIGNAL_RUN_FIRST, TYPE_NONE, (str,)),
         "game_ended":    (SIGNAL_RUN_FIRST, TYPE_NONE, (int,)),
+        "game_terminated":    (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
+        "game_resumed":  (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
         "action_error":  (SIGNAL_RUN_FIRST, TYPE_NONE, (object, int))
     }
     
-    def __init__ (self, timemodel=None):
+    def __init__ (self, timemodel=None, variant=NormalChess):
         GObject.__init__(self)
+
+        self.variant = variant
+        self.boards = [variant.board(setup=True)]
         
-        self.boards = [Board(setup=True)]
         self.moves = []
         self.players = []
         
@@ -44,6 +77,8 @@ class GameModel (GObject, PooledThread):
         self.reason = UNKNOWN_REASON
         
         self.timemodel = timemodel
+
+        self.connections = defaultdict(list)  # mainly for IC subclasses
         
         today = datetime.date.today()
         self.tags = {
@@ -56,24 +91,38 @@ class GameModel (GObject, PooledThread):
         }
         
         # Keeps track of offers, so that accepts can be spotted
-        self.offerMap = {}
+        self.offers = {}
         # True if the game has been changed since last save
         self.needsSave = False
         # The uri the current game was loaded from, or None if not a loaded game
         self.uri = None
         
+        self.hintEngineSupportsVariant = False
+        self.spyEngineSupportsVariant = False
+        
         self.spectactors = {}
         
         self.applyingMoveLock = RLock()
+        self.undoLock = RLock()
+        self.undoQueue = Queue.Queue()
+    
+    def __repr__ (self):
+        s = ""
+        s += "ply=%s" % str(self.ply)
+        s += ", status=%s, reason=%s" % (str(self.status), str(self.reason))
+        s += ", players=%s" % str(self.players)
+        if len(self.boards) > 0:
+            s += ", %s" % self.boards[-1]
+        return s
     
     def setPlayers (self, players):
         assert self.status == WAITING_TO_START
         self.players = players
         for player in self.players:
-            player.connect("offer", self.offerRecieved)
-            player.connect("withdraw", self.withdrawRecieved)
-            player.connect("decline", self.declineRecieved)
-            player.connect("accept", self.acceptRecieved)
+            self.connections[player].append(player.connect("offer", self.offerRecieved))
+            self.connections[player].append(player.connect("withdraw", self.withdrawRecieved))
+            self.connections[player].append(player.connect("decline", self.declineRecieved))
+            self.connections[player].append(player.connect("accept", self.acceptRecieved))
     
     def setSpectactors (self, spectactors):
         assert self.status == WAITING_TO_START
@@ -92,43 +141,65 @@ class GameModel (GObject, PooledThread):
     lowply = property(_get_lowest_ply)
     
     def _get_curplayer (self):
-        return self.players[self.boards[-1].color]
+        try:
+            return self.players[self.getBoardAtPly(self.ply).color]
+        except IndexError:
+            log.error("%s %s\n" % (self.players, self.getBoardAtPly(self.ply).color))
+            raise
     curplayer = property(_get_curplayer)
     
     def _plyToIndex (self, ply):
-        index = ply - self.boards[0].ply
+        index = ply - self.lowply
         if index < 0:
-            raise IndexError, "%s < %s" % (ply, self.boards[0].ply)
+            raise IndexError, "%s < %s\n" % (ply, self.lowply)
         return index
     
     def getBoardAtPly (self, ply):
         try:
             return self.boards[self._plyToIndex(ply)]
         except:
-            log.debug("%d\t%d\t%d\t%d\n" % (self.lowply, ply, self.ply, len(self.boards)))
+            log.error("%d\t%d\t%d\t%d\n" % (self.lowply, ply, self.ply, len(self.boards)))
             raise
     
     def getMoveAtPly (self, ply):
-        return self.moves[self._plyToIndex(ply)]
+        try:
+            return self.moves[self._plyToIndex(ply)]
+        except IndexError:
+            log.error("%d\t%d\t%d\t%d\n" % (self.lowply, ply, self.ply, len(self.moves)))
+            raise
     
     ############################################################################
     # Offer management                                                         #
     ############################################################################
     
     def offerRecieved (self, player, offer):
+        log.debug("GameModel.offerRecieved: offerer=%s %s\n" % (repr(player), offer))
         if player == self.players[WHITE]:
             opPlayer = self.players[BLACK]
         else: opPlayer = self.players[WHITE]
         
-        if offer.offerType == HURRY_REQUEST:
+        if self.status not in UNFINISHED_STATES and offer.type in INGAME_ACTIONS:
+            player.offerError(offer, ACTION_ERROR_REQUIRES_UNFINISHED_GAME)
+        
+        elif offer.type == RESUME_OFFER and self.status in (DRAW, WHITEWON,BLACKWON) and \
+                self.reason in UNRESUMEABLE_REASONS:
+            player.offerError(offer, ACTION_ERROR_UNRESUMEABLE_POSITION)
+        
+        elif offer.type == RESUME_OFFER and self.status != PAUSED:
+            player.offerError(offer, ACTION_ERROR_RESUME_REQUIRES_PAUSED)
+        
+        elif offer.type == HURRY_ACTION:
             opPlayer.hurry()
         
-        elif offer.offerType == RESIGNATION:
+        elif offer.type == CHAT_ACTION:
+            opPlayer.putMessage(offer.param)
+        
+        elif offer.type == RESIGNATION:
             if player == self.players[WHITE]:
                 self.end(BLACKWON, WON_RESIGN)
             else: self.end(WHITEWON, WON_RESIGN)
         
-        elif offer.offerType == FLAG_CALL:
+        elif offer.type == FLAG_CALL:
             if not self.timemodel:
                 player.offerError(offer, ACTION_ERROR_NO_CLOCK)
                 return
@@ -140,6 +211,11 @@ class GameModel (GObject, PooledThread):
             if self.timemodel.getPlayerTime (opcolor) <= 0:
                 if self.timemodel.getPlayerTime (1-opcolor) <= 0:
                     self.end(DRAW, DRAW_CALLFLAG)
+                elif not playerHasMatingMaterial(self.boards[-1], (1-opcolor)):
+                    if opcolor == WHITE:
+                        self.end(DRAW, DRAW_BLACKINSUFFICIENTANDWHITETIME)
+                    else:
+                        self.end(DRAW, DRAW_WHITEINSUFFICIENTANDBLACKTIME)
                 else:
                     if player == self.players[WHITE]:
                         self.end(WHITEWON, WON_CALLFLAG)
@@ -148,63 +224,76 @@ class GameModel (GObject, PooledThread):
             else:
                 player.offerError(offer, ACTION_ERROR_NOT_OUT_OF_TIME)
         
-        elif offer.offerType == DRAW_OFFER and isClaimableDraw(self.boards[-1]):
+        elif offer.type == DRAW_OFFER and isClaimableDraw(self.boards[-1]):
             reason = getStatus(self.boards[-1])[1]
             self.end(DRAW, reason)
         
-        elif offer.offerType in OFFERS:
-            if offer.offerType == TAKEBACK_OFFER and offer.param < self.lowply:
-                player.offerError(offer, ACTION_ERROR_TO_LARGE_UNDO)
-                return
-            if offer not in self.offerMap:
-                self.offerMap[offer] = player
+        elif offer.type == TAKEBACK_OFFER and offer.param < self.lowply:
+            player.offerError(offer, ACTION_ERROR_TOO_LARGE_UNDO)
+        
+        elif offer.type == TAKEBACK_OFFER and self.status != RUNNING and \
+                (self.status not in UNDOABLE_STATES or self.reason not in UNDOABLE_REASONS):
+            player.offerError(offer, ACTION_ERROR_GAME_ENDED)
+        
+        elif offer.type in OFFERS:
+            if offer not in self.offers:
+                log.debug("GameModel.offerRecieved: doing %s.offer(%s)\n" % \
+                    (repr(opPlayer), offer))
+                self.offers[offer] = player
                 opPlayer.offer(offer)
             # If we updated an older offer, we want to delete the old one
-            for of in self.offerMap.keys():
-                if offer.offerType == of.offerType and offer != of:
-                    del self.offerMap[of]
+            for offer_ in self.offers.keys():
+                if offer.type == offer_.type and offer != offer_:
+                    del self.offers[offer_]
     
     def withdrawRecieved (self, player, offer):
+        log.debug("GameModel.withdrawRecieved: withdrawer=%s %s\n" % \
+            (repr(player), offer))
         if player == self.players[WHITE]:
             opPlayer = self.players[BLACK]
         else: opPlayer = self.players[WHITE]
         
-        if offer in self.offerMap and self.offerMap[offer] == player:
-            del self.offerMap[offer]
+        if offer in self.offers and self.offers[offer] == player:
+            del self.offers[offer]
             opPlayer.offerWithdrawn(offer)
         else:
             player.offerError(offer, ACTION_ERROR_NONE_TO_WITHDRAW)
     
     def declineRecieved (self, player, offer):
+        log.debug("GameModel.declineRecieved: decliner=%s %s\n" % (repr(player), offer))
         if player == self.players[WHITE]:
             opPlayer = self.players[BLACK]
         else: opPlayer = self.players[WHITE]
         
-        if offer in self.offerMap and self.offerMap[offer] == opPlayer:
-            del self.offerMap[offer]
+        if offer in self.offers and self.offers[offer] == opPlayer:
+            del self.offers[offer]
+            log.debug("GameModel.declineRecieved: declining %s\n" % offer)
             opPlayer.offerDeclined(offer)
         else:
             player.offerError(offer, ACTION_ERROR_NONE_TO_DECLINE)
     
     def acceptRecieved (self, player, offer):
+        log.debug("GameModel.acceptRecieved: accepter=%s %s\n" % (repr(player), offer))
         if player == self.players[WHITE]:
             opPlayer = self.players[BLACK]
         else: opPlayer = self.players[WHITE]
         
-        if offer in self.offerMap and self.offerMap[offer] == opPlayer:
-            if offer.offerType == DRAW_OFFER:
+        if offer in self.offers and self.offers[offer] == opPlayer:
+            if offer.type == DRAW_OFFER:
                 self.end(DRAW, DRAW_AGREE)
-            elif offer.offerType == TAKEBACK_OFFER:
+            elif offer.type == TAKEBACK_OFFER:
+                log.debug("GameModel.acceptRecieved: undoMoves(%s)\n" % \
+                    (self.ply - offer.param))
                 self.undoMoves(self.ply - offer.param)
-            elif offer.offerType == ADJOURN_OFFER:
+            elif offer.type == ADJOURN_OFFER:
                 self.end(ADJOURNED, ADJOURNED_AGREEMENT)
-            elif offer.offerType == ABORT_OFFER:
+            elif offer.type == ABORT_OFFER:
                 self.end(ABORTED, ABORTED_AGREEMENT)
-            elif offer.offerType == PAUSE_OFFER:
+            elif offer.type == PAUSE_OFFER:
                 self.pause()
-            elif offer.offerType == RESUME_OFFER:
+            elif offer.type == RESUME_OFFER:
                 self.resume()
-            del self.offerMap[offer]
+            del self.offers[offer]
         else:
             player.offerError(offer, ACTION_ERROR_NONE_TO_ACCEPT)
     
@@ -214,13 +303,14 @@ class GameModel (GObject, PooledThread):
     
     def loadAndStart (self, uri, loader, gameno, position):
         assert self.status == WAITING_TO_START
-        
+
         uriIsFile = type(uri) != str
         if not uriIsFile:
             chessfile = loader.load(protoopen(uri))
-        else: chessfile = loader.load(uri)
+        else: 
+            chessfile = loader.load(uri)
         
-        self.emit("game_loading")
+        self.emit("game_loading", uri)
         try:
             chessfile.loadToModel(gameno, position, self)
         #Postpone error raising to make games loadable to the point of the error
@@ -236,9 +326,9 @@ class GameModel (GObject, PooledThread):
         
         if self.status == RUNNING:
             for player in self.players:
-                player.setBoard(self)
+                player.setOptionInitialBoard(self)
             for spectactor in self.spectactors.values():
-                spectactor.setBoard(self)
+                spectactor.setOptionInitialBoard(self)
             
             if self.timemodel:
                 self.timemodel.setMovingColor(self.boards[-1].color)
@@ -281,193 +371,228 @@ class GameModel (GObject, PooledThread):
         # Avoid racecondition when self.start is called while we are in self.end
         if self.status != WAITING_TO_START:
             return
-        
         self.status = RUNNING
+        
+        for player in self.players + self.spectactors.values():
+            player.start()
+        
         self.emit("game_started")
         
-        while self.status in (PAUSED, RUNNING):
+        while self.status in (PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON):
             curColor = self.boards[-1].color
             curPlayer = self.players[curColor]
             
             if self.timemodel:
+                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: updating %s's time\n" % \
+                    (id(self), str(self.players), str(self.ply), str(curPlayer)))
                 curPlayer.updateTime(self.timemodel.getPlayerTime(curColor),
                                      self.timemodel.getPlayerTime(1-curColor))
             
             try:
-                move = curPlayer.makeMove(self)
+                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: calling %s.makeMove()\n" % \
+                    (id(self), str(self.players), self.ply, str(curPlayer)))
+                if self.ply > self.lowply:
+                    move = curPlayer.makeMove(self.boards[-1],
+                                              self.moves[-1],
+                                              self.boards[-2])
+                else: move = curPlayer.makeMove(self.boards[-1], None, None)
+                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from %s\n" % \
+                    (id(self), str(self.players), self.ply, move, str(curPlayer)))
             except PlayerIsDead, e:
                 if self.status in (WAITING_TO_START, PAUSED, RUNNING):
                     stringio = cStringIO.StringIO()
                     traceback.print_exc(file=stringio)
                     error = stringio.getvalue()
-                    log.error("A Player died:%s\n%s" % (e, error), curPlayer)
+                    log.error("GameModel.run: A Player died: player=%s error=%s\n%s" % (curPlayer, error, e))
                     if curColor == WHITE:
                         self.kill(WHITE_ENGINE_DIED)
                     else: self.kill(BLACK_ENGINE_DIED)
                 break
             except TurnInterrupt:
+                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: TurnInterrupt\n" % \
+                    (id(self), str(self.players), self.ply))
                 continue
             
+            log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: acquiring self.applyingMoveLock\n" % \
+                (id(self), str(self.players), self.ply))
             self.applyingMoveLock.acquire()
             try:
+                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: applying move=%s\n" % \
+                    (id(self), str(self.players), self.ply, str(move)))
                 self.needsSave = True
                 newBoard = self.boards[-1].move(move)
                 self.boards.append(newBoard)
                 self.moves.append(move)
+                
                 if self.timemodel:
                     self.timemodel.tap()
-                if not self.checkStatus():
-                    break
+                    
+                self.checkStatus()
+                
                 self.emit("game_changed")
                 
                 for spectactor in self.spectactors.values():
-                    model = GameModel()
-                    model.boards = [board.clone() for board in self.boards]
-                    model.moves = [move for move in self.moves]
-                    model.players = self.players
-                    model.status = self.status
-                    model.reason = self.reason
-                    spectactor.makeMove(model)
+                    spectactor.putMove(self.boards[-1], self.moves[-1], self.boards[-2])
             finally:
+                log.debug("GameModel.run: releasing self.applyingMoveLock\n")
                 self.applyingMoveLock.release()
     
     def checkStatus (self):
-        if self.status not in (WAITING_TO_START, PAUSED, RUNNING):
-            return False
+        log.debug("GameModel.checkStatus:\n")
         status, reason = getStatus(self.boards[-1])
-        if status not in (WAITING_TO_START, PAUSED, RUNNING):
-            self.status = status
-            self.emit("game_changed")
-            self.status = RUNNING # self.end only accepts ending if running
-            self.end(status, reason)
-            return False
         
-        return True
+        if status != RUNNING and self.status in (WAITING_TO_START, PAUSED, RUNNING):
+            if not (status == DRAW and reason in (DRAW_REPITITION, DRAW_50MOVES)):
+                self.end(status, reason)
+                return
+        
+        if status != self.status and self.status in UNDOABLE_STATES \
+                and self.reason in UNDOABLE_REASONS:
+            self.__resume()
+            self.status = status
+            self.reason = UNKNOWN_REASON
+            self.emit("game_unended")
+   
+    def __pause (self):
+        for player in self.players:
+            player.pause()
+        try:
+            for spectactor in self.spectactors.values():
+                spectactor.pause()
+        except NotImplementedError:
+            pass
+        if self.timemodel:
+            self.timemodel.pause()
     
+    @inthread
     def pause (self):
         """ Players will raise NotImplementedError if they doesn't support
             pause. Spectactors will be ignored. """
         
-        glock.release()
         self.applyingMoveLock.acquire()
-        glock.acquire()
         try:
-            for player in self.players:
-                player.pause()
-            
-            try:
-                for spectactor in self.spectactors.values():
-                    spectactor.pause()
-            except NotImplementedError:
-                pass
-            
-            if self.timemodel:
-                self.timemodel.pause()
-            
+            self.__pause()
             self.status = PAUSED
         finally:
-            glock.release()
             self.applyingMoveLock.release()
-            glock.acquire()
     
-    def resume (self):
-        
-        glock.release()
-        self.applyingMoveLock.acquire()
-        glock.acquire()
+    def __resume (self):
+        for player in self.players:
+            player.resume()
         try:
-            for player in self.players:
-                player.resume()
-            
-            try:
-                for spectactor in self.spectactors.values():
-                    spectactor.resume()
-            except NotImplementedError:
-                pass
-            
-            if self.timemodel:
-                self.timemodel.resume()
-            
+            for spectactor in self.spectactors.values():
+                spectactor.resume()
+        except NotImplementedError:
+            pass
+        if self.timemodel:
+            self.timemodel.resume()
+        self.emit("game_resumed")
+    
+    @inthread
+    def resume (self):
+        self.applyingMoveLock.acquire()
+        try:
+            self.__resume()
             self.status = RUNNING
         finally:
-            glock.release()
             self.applyingMoveLock.release()
     
     def end (self, status, reason):
-        if not self.status in (WAITING_TO_START, PAUSED, RUNNING):
+        if self.status not in UNFINISHED_STATES:
+            log.log("GameModel.end: Can't end a game that's already ended: %s %s\n" % (status, reason))
             return
+        if self.status not in (WAITING_TO_START, PAUSED, RUNNING):
+            self.needsSave = True
         
-        log.debug("Ending a game with status %d for reason %d\n%s" % (status, reason,
-            "".join(traceback.format_list(traceback.extract_stack())).strip()))
+        #log.debug("Ending a game with status %d for reason %d\n%s" % (status, reason,
+        #    "".join(traceback.format_list(traceback.extract_stack())).strip()))
+        log.debug("GameModel.end: players=%s, self.ply=%s: Ending a game with status %d for reason %d\n" % \
+            (repr(self.players), str(self.ply), status, reason))
         self.status = status
+        self.reason = reason
         
-        for player in self.players:
-            player.end(self.status, self.reason)
-        
-        for spectactor in self.spectactors.values():
-            spectactor.end(self.status, self.reason)
-        
-        if self.timemodel:
-            self.timemodel.pause()
-        
-        self.needsSave = True
         self.emit("game_ended", reason)
+        
+        self.__pause()
     
     def kill (self, reason):
-        if not self.status in (WAITING_TO_START, PAUSED, RUNNING):
-            return
+        log.debug("GameModel.kill: players=%s, self.ply=%s: Killing a game for reason %d\n%s" % \
+            (repr(self.players), str(self.ply), reason,
+             "".join(traceback.format_list(traceback.extract_stack())).strip()))
         
-        log.debug("Killing a game for reason %d\n%s" % (reason,
-            "".join(traceback.format_list(traceback.extract_stack())).strip()))
         self.status = KILLED
         self.reason = reason
         
         for player in self.players:
-            player.kill(reason)
+            player.end(self.status, reason)
         
         for spectactor in self.spectactors.values():
-            spectactor.kill(reason)
+            spectactor.end(self.status, reason)
         
         if self.timemodel:
-            self.timemodel.pause()
+            self.timemodel.end()
         
         self.emit("game_ended", reason)
     
+    def terminate (self):
+        
+        if self.status != KILLED:
+            #self.resume()
+            for player in self.players:
+                player.end(self.status, self.reason)
+            
+            for spectactor in self.spectactors.values():
+                spectactor.end(self.status, self.reason)
+            
+            if self.timemodel:
+                self.timemodel.end()
+        
+        self.emit("game_terminated")
+    
     ############################################################################
     # Other stuff                                                              #
     ############################################################################
     
+    @inthread
+    @undolocked
     def undoMoves (self, moves):
-        """ Will push back one full move by calling the undo methods of players
-            and spectactors. If they raise NotImplementedError we'll try to call
-            setBoard instead """
-        
-        # We really shouldn't do this at the same time we are applying a move
-        # On the other hand it shouldn't matter to undo a move while a player is
-        # thinking, as the player should be smart enough.
-        
-        self.emit("moves_undoing", moves)
+        """ Undo and remove moves number of moves from the game history from
+            the GameModel, players, and any spectactors """
+        if self.ply < 1 or moves < 1: return
+        if self.ply - moves < 0:
+            # There is no way in the current threaded/asynchronous design
+            # for the GUI to know that the number of moves it requests to takeback
+            # will still be valid once the undo is actually processed. So, until
+            # we either add some locking or get a synchronous design, we quietly
+            # "fix" the takeback request rather than cause AssertionError or IndexError  
+            moves = 1
         
+        log.debug("GameModel.undoMoves: players=%s, self.ply=%s, moves=%s, board=%s" % \
+            (repr(self.players), self.ply, moves, self.boards[-1]))
+        log.debug("GameModel.undoMoves: acquiring self.applyingMoveLock\n")
         self.applyingMoveLock.acquire()
+        log.debug("GameModel.undoMoves: self.applyingMoveLock acquired\n")
         try:
+            self.emit("moves_undoing", moves)
             self.needsSave = True
             
             del self.boards[-moves:]
             del self.moves[-moves:]
             
-            for player in list(self.players) + list(self.spectactors.values()):
-                try:
-                    player.undoMoves(moves, self)
-                except NotImplementedError:
-                    # If the player doesn't support undoing, we might be able to
-                    # simply "load" the new last board
-                    player.setBoard(self)
+            for player in self.players:
+                player.playerUndoMoves(moves, self)
+            for spectator in self.spectactors.values():
+                spectator.spectatorUndoMoves(moves, self)
             
+            log.debug("GameModel.undoMoves: undoing timemodel\n")
             if self.timemodel:
                 self.timemodel.undoMoves(moves)
+            
+            self.checkStatus()
         finally:
+            log.debug("GameModel.undoMoves: releasing self.applyingMoveLock\n")
             self.applyingMoveLock.release()
-    
+        
     def isChanged (self):
         if self.ply == 0:
             return False
