@@ -5,8 +5,7 @@
 ################################################################################
 
 NAME = "PyChess"
-VERSION = "0.8.2"
-VERSION_NAME = "Philidor"
+ENGINES_XML_API_VERSION = "0.10.1"
 
 ################################################################################
 # Player info                                                                  #
@@ -31,28 +30,79 @@ WAITING_TO_START, PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON, KILLED, \
     ADJOURNED, ABORTED, UNKNOWN_STATE = range(10)
 reprResult = ["*", "*", "*", "1/2-1/2", "1-0", "0-1", "?", "*", "?", "?"]
 
+UNDOABLE_STATES = (DRAW, WHITEWON, BLACKWON)
+UNFINISHED_STATES = (WAITING_TO_START, PAUSED, RUNNING, UNKNOWN_STATE)
+
+# Game types
+TYPE_BLITZ, TYPE_STANDARD, TYPE_LIGHTNING, TYPE_WILD, TYPE_BUGHOUSE, \
+    TYPE_CRAZYHOUSE, TYPE_SUICIDE, TYPE_LOSERS, TYPE_ATOMIC, TYPE_UNTIMED = range(10)
+isLightning = lambda secs, incr: secs + incr * 40 < 3*60
+isBlitz = lambda secs, incr: secs + incr * 40 < 15*60
+
+typeName = (_("Blitz"), _("Standard"), _("Lightning"), _("Wild"), _("Bughouse"),
+            _("Crazyhouse"), _("Suicide"), _("Losers"), _("Atomic"))
+
+# Chess variants
+NORMALCHESS, CORNERCHESS, SHUFFLECHESS, FISCHERRANDOMCHESS, RANDOMCHESS, \
+ASYMMETRICRANDOMCHESS, UPSIDEDOWNCHESS, PAWNSPUSHEDCHESS, PAWNSPASSEDCHESS, \
+LOSERSCHESS, PAWNODDSCHESS, KNIGHTODDSCHESS, ROOKODDSCHESS, \
+QUEENODDSCHESS = range(14)
+
+# Chess variant groups
+VARIANTS_BLINDFOLD, VARIANTS_ODDS, VARIANTS_SHUFFLE, VARIANTS_OTHER = range(4)
+
 # Action errors
 ACTION_ERROR_NO_CLOCK, ACTION_ERROR_NOT_OUT_OF_TIME, \
     ACTION_ERROR_CLOCK_NOT_STARTED, ACTION_ERROR_SWITCH_UNDERWAY, \
-    ACTION_ERROR_CLOCK_NOT_PAUSED, ACTION_ERROR_TO_LARGE_UNDO, \
+    ACTION_ERROR_CLOCK_NOT_PAUSED, ACTION_ERROR_TOO_LARGE_UNDO, \
     ACTION_ERROR_NONE_TO_ACCEPT, ACTION_ERROR_NONE_TO_WITHDRAW, \
-    ACTION_ERROR_NONE_TO_DECLINE = range(9)
+    ACTION_ERROR_NONE_TO_DECLINE, ACTION_ERROR_GAME_ENDED, \
+    ACTION_ERROR_REQUIRES_UNFINISHED_GAME, \
+    ACTION_ERROR_UNRESUMEABLE_POSITION, \
+    ACTION_ERROR_RESUME_REQUIRES_PAUSED, \
+    ACTION_ERROR_UNSUPPORTED_FICS_WHEN_GAME_FINISHED = range(14)
 
 # Game state reasons
 ABORTED_ADJUDICATION, ABORTED_AGREEMENT, ABORTED_COURTESY, ABORTED_EARLY, \
-    ABORTED_SERVER_SHUTDOWN, \
+    ABORTED_SERVER_SHUTDOWN, ADJOURNED_COURTESY, \
 ADJOURNED_AGREEMENT, ADJOURNED_LOST_CONNECTION, ADJOURNED_SERVER_SHUTDOWN, \
 DRAW_50MOVES, DRAW_ADJUDICATION, DRAW_AGREE, DRAW_CALLFLAG, DRAW_INSUFFICIENT, \
     DRAW_LENGTH, DRAW_REPITITION, DRAW_STALEMATE, \
+DRAW_BLACKINSUFFICIENTANDWHITETIME, DRAW_WHITEINSUFFICIENTANDBLACKTIME, \
 WON_ADJUDICATION, WON_CALLFLAG, WON_DISCONNECTION, WON_MATE, WON_RESIGN, \
-WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, UNKNOWN_REASON = range(24)
+    WON_NOMATERIAL, \
+WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, DISCONNECTED, UNKNOWN_REASON = range(29)
+
+UNDOABLE_REASONS = (DRAW_50MOVES, DRAW_INSUFFICIENT, DRAW_LENGTH,
+                    DRAW_REPITITION, DRAW_STALEMATE, DRAW_AGREE, DRAW_CALLFLAG, \
+                    DRAW_BLACKINSUFFICIENTANDWHITETIME, \
+                    DRAW_WHITEINSUFFICIENTANDBLACKTIME, \
+                    WON_MATE, WON_NOMATERIAL, WON_CALLFLAG, WON_RESIGN)
+
+UNRESUMEABLE_REASONS = (DRAW_50MOVES, DRAW_INSUFFICIENT, DRAW_LENGTH, \
+                        DRAW_REPITITION, DRAW_STALEMATE, WON_MATE, WON_NOMATERIAL)
 
 # Player actions
-RESIGNATION, FLAG_CALL, DRAW_OFFER, ABORT_OFFER, ADJOURN_OFFER, PAUSE_OFFER, \
-    RESUME_OFFER, SWITCH_OFFER, TAKEBACK_OFFER, TAKEBACK_FORCE, MATCH_OFFER, \
-    HURRY_REQUEST = range(12)
-OFFERS = set([DRAW_OFFER, ABORT_OFFER, ADJOURN_OFFER, PAUSE_OFFER, \
-    RESUME_OFFER, SWITCH_OFFER, TAKEBACK_OFFER, MATCH_OFFER])
+RESIGNATION = "resignation"
+FLAG_CALL = "flag call"
+DRAW_OFFER = "draw offer"
+ABORT_OFFER = "abort offer"
+ADJOURN_OFFER = "adjourn offer"
+PAUSE_OFFER = "pause offer"
+RESUME_OFFER = "resume offer"
+SWITCH_OFFER = "switch offer"
+TAKEBACK_OFFER = "takeback offer"
+MATCH_OFFER = "match offer"
+HURRY_ACTION = "hurry action"
+CHAT_ACTION = "chat action"
+
+ACTIONS = (RESIGNATION, FLAG_CALL, DRAW_OFFER, ABORT_OFFER, ADJOURN_OFFER, \
+           PAUSE_OFFER, RESUME_OFFER, SWITCH_OFFER, TAKEBACK_OFFER, \
+           MATCH_OFFER, HURRY_ACTION, CHAT_ACTION)
+OFFERS = (DRAW_OFFER, ABORT_OFFER, ADJOURN_OFFER, PAUSE_OFFER, \
+          RESUME_OFFER, SWITCH_OFFER, TAKEBACK_OFFER, MATCH_OFFER)
+INGAME_ACTIONS = (RESIGNATION, FLAG_CALL, DRAW_OFFER, ABORT_OFFER, \
+                  ADJOURN_OFFER, PAUSE_OFFER, SWITCH_OFFER, HURRY_ACTION)
 
 # A few nice to have boards
 FEN_EMPTY = "8/8/8/8/8/8/8/8 w KQkq - 0 1"
@@ -62,7 +112,7 @@ FEN_START = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"
 # Search values                                                                #
 ################################################################################
 
-hashfALPHA, hashfBETA, hashfEXACT = range(3)
+hashfALPHA, hashfBETA, hashfEXACT, hashfBAD = range(4)
 
 # Engine modes
 NORMAL, ANALYZING, INVERSE_ANALYZING = range(3)
@@ -143,7 +193,10 @@ for cord, name in enumerate(reprCord):
 IC_CONNECTED, IC_DISCONNECTED = range(2)
 
 IC_POS_ISOLATED, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, \
-IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, IC_POS_OBSERVING = range(6)
+IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, IC_POS_OBSERVING, IC_POS_INITIAL = range(7)
+
+IC_STATUS_PLAYING, IC_STATUS_ACTIVE, IC_STATUS_BUSY, \
+    IC_STATUS_OFFLINE = range(4)
 
 ################################################################################
 # User interface                                                               #
@@ -158,21 +211,10 @@ SOUND_MUTE, SOUND_BEEP, SOUND_SELECT, SOUND_URI = range(4)
 # Brush types. Send piece object for Piece brush
 CLEAR, ENPAS = range(2)
 
-import sys, os
-if sys.platform == "win32":
-    username = os.environ["USERNAME"]
-else:
-    from os import getuid
-    from pwd import getpwuid
-    userdata = getpwuid(getuid())
-    i = userdata.pw_gecos.find(',')
-    if i >= 0:
-        username = userdata.pw_gecos[i:]
-    else: username = userdata.pw_name
-    del userdata, i
-
 ################################################################################
-# Subprocess types                                                             #
+# Subprocess                                                                   #
 ################################################################################
 
 SUBPROCESS_PTY, SUBPROCESS_SUBPROCESS, SUBPROCESS_FORK = range(3)
+
+LOG_DEBUG, LOG_LOG, LOG_WARNING, LOG_ERROR = range(4)
