@@ -6,6 +6,7 @@ from ldata import *
 from attack import isAttacked
 from bitboard import *
 from threading import RLock
+from copy import deepcopy
 
 ################################################################################
 # Zobrit hashing 32 bit implementation                                         #
@@ -48,18 +49,16 @@ STRICT_FEN = False
 ################################################################################
 
 class LBoard:
-    def __init__ (self):
+    def __init__ (self, variant):
+        self.variant = variant
         self._reset()
     
     def _reset (self):
-        self.blocker45 = 0
-        self.blocker315 = 0
-        self.blocker = 0
-        self.blocker90 = 0
+        self.blocker = createBoard(0)
         
-        self.friends = [0]*2
+        self.friends = [createBoard(0)]*2
         self.kings = [-1]*2
-        self.boards = [[0]*7 for i in range(2)]
+        self.boards = [[createBoard(0)]*7 for i in range(2)]
         
         self.enpassant = -1
         self.color = WHITE
@@ -72,12 +71,6 @@ class LBoard:
         
         self.arBoard = array("B", [0]*64)
         
-        # This lock is intended to be used anywhere you need to first add some
-        # moves and then pop them, to be sure the board is in the correct state
-        # if another thread looks at it at the same time.
-        # Examples are printing a list of sanmoves or checking for check.
-        self.lock = RLock()
-        
         self.hash = 0
         self.pawnhash = 0
         
@@ -91,6 +84,14 @@ class LBoard:
         #  fifty:      A counter for the fifty moves rule                      #
         ########################################################################
         self.history = []
+
+        # initial cords of rooks and kings for castling in Chess960
+        if self.variant == FISCHERRANDOMCHESS:
+            self.ini_kings = [None, None]
+            self.ini_rooks = [[None, None], [None, None]]
+        else:
+            self.ini_kings = [E1, E8]
+            self.ini_rooks = [[A1, H1], [A8, H8]]
     
     def applyFen (self, fenstr):
         """ Applies the fenstring to the board.
@@ -159,7 +160,15 @@ class LBoard:
                     piece = reprSign.index(char.upper())
                     self._addPiece(cord, piece, color)
                     cord += 1
-        
+
+            if self.variant == FISCHERRANDOMCHESS:
+                # Save ranks fo find outermost rooks
+                # if KkQq was used in castling rights
+                if r == 0:
+                    rank8 = rank
+                elif r == 7:
+                    rank1 = rank
+
         # Parse active color field
         
         if colChr.lower() == "w":
@@ -167,19 +176,53 @@ class LBoard:
         else: self.setColor (BLACK)
         
         # Parse castling availability
-        
+
         castling = 0
         for char in castChr:
-            if char == "K":
-                castling |= W_OO
-            elif char == "Q":
-                castling |= W_OOO
-            elif char == "k":
-                castling |= B_OO
-            elif char == "q":
-                castling |= B_OOO
+            if self.variant == FISCHERRANDOMCHESS:
+                if char in reprFile:
+                    if char < reprCord[self.kings[BLACK]][0]:
+                        castling |= B_OOO
+                        self.ini_rooks[1][0] = reprFile.index(char) + 56
+                    else:
+                        castling |= B_OO
+                        self.ini_rooks[1][1] = reprFile.index(char) + 56
+                    self.ini_kings[BLACK] = self.kings[BLACK]
+                elif char in [c.upper() for c in reprFile]:
+                    if char < reprCord[self.kings[WHITE]][0].upper():
+                        castling |= W_OOO
+                        self.ini_rooks[0][0] = reprFile.index(char.lower())
+                    else:
+                        castling |= W_OO
+                        self.ini_rooks[0][1] = reprFile.index(char.lower())
+                    self.ini_kings[WHITE] = self.kings[WHITE]
+                elif char == "K":
+                    castling |= W_OO
+                    self.ini_rooks[0][1] = rank1.rfind('R')
+                    self.ini_kings[WHITE] = self.kings[WHITE]
+                elif char == "Q":
+                    castling |= W_OOO
+                    self.ini_rooks[0][0] = rank1.find('R')
+                    self.ini_kings[WHITE] = self.kings[WHITE]
+                elif char == "k":
+                    castling |= B_OO
+                    self.ini_rooks[1][1] = rank8.rfind('r')
+                    self.ini_kings[BLACK] = self.kings[BLACK]
+                elif char == "q":
+                    castling |= B_OOO
+                    self.ini_rooks[1][0] = rank8.find('r')
+                    self.ini_kings[BLACK] = self.kings[BLACK]
+            else:
+                if char == "K":
+                    castling |= W_OO
+                elif char == "Q":
+                    castling |= W_OOO
+                elif char == "k":
+                    castling |= B_OO
+                elif char == "q":
+                    castling |= B_OOO
         self.setCastling(castling)
-        
+
         # Parse en passant target sqaure
         
         if epChr == "-":
@@ -188,7 +231,7 @@ class LBoard:
         
         # Parse halfmove clock field
         
-        self.fifty = int(fiftyChr)
+        self.fifty = max(int(fiftyChr),0)
         
         # Parse fullmove number
         
@@ -213,11 +256,10 @@ class LBoard:
     def _addPiece (self, cord, piece, color):
         self.boards[color][piece] = \
                 setBit(self.boards[color][piece], cord)
-        self.blocker90 = setBit(self.blocker90, r90[cord])
-        self.blocker45 = setBit(self.blocker45, r45[cord])
-        self.blocker315 = setBit(self.blocker315, r315[cord])
         
         if piece == PAWN:
+            #assert not (color == WHITE and cord > 55)
+            #assert not (color == BLACK and cord < 8)
             self.pawnhash ^= pieceHashes[color][PAWN][cord]
         elif piece == KING:
             self.kings[color] = cord
@@ -228,9 +270,6 @@ class LBoard:
     def _removePiece (self, cord, piece, color):
         self.boards[color][piece] = \
                 clearBit(self.boards[color][piece], cord)
-        self.blocker90 = clearBit(self.blocker90, r90[cord])
-        self.blocker45 = clearBit(self.blocker45, r45[cord])
-        self.blocker315 = clearBit(self.blocker315, r315[cord])
         
         if piece == PAWN:
             self.pawnhash ^= pieceHashes[color][PAWN][cord]
@@ -298,12 +337,17 @@ class LBoard:
         
         # Capture
         if tpiece != EMPTY:
-            self._removePiece(tcord, tpiece, opcolor)
+            if self.variant == FISCHERRANDOMCHESS:
+                # don't capture _our_ piece when castling king steps on rook!
+                if flag not in (KING_CASTLE, QUEEN_CASTLE):
+                    self._removePiece(tcord, tpiece, opcolor)
+            else:
+                self._removePiece(tcord, tpiece, opcolor)
         
         if fpiece == PAWN:
 
             if flag == ENPASSANT:
-                takenPawnC = self.enpassant + (self.color == WHITE and -8 or 8)
+                takenPawnC = tcord + (self.color == WHITE and -8 or 8)
                 self._removePiece (takenPawnC, PAWN, opcolor)
                 
             elif flag in PROMOTIONS:
@@ -317,12 +361,31 @@ class LBoard:
         
         if flag in (KING_CASTLE, QUEEN_CASTLE):
             if flag == QUEEN_CASTLE:
-                rookf = fcord - 4
-                rookt = fcord - 1
+                if self.variant == FISCHERRANDOMCHESS:
+                    if self.color == WHITE:
+                        rookf = self.ini_rooks[0][0]
+                        rookt = D1
+                    else:
+                        rookf = self.ini_rooks[1][0]
+                        rookt = D8
+                    # don't move our rook yet
+                else:
+                    rookf = fcord - 4
+                    rookt = fcord - 1
+                    self._move (rookf, rookt, ROOK, self.color)
             else:
-                rookf = fcord + 3
-                rookt = fcord + 1
-            self._move (rookf, rookt, ROOK, self.color)
+                if self.variant == FISCHERRANDOMCHESS:
+                    if self.color == WHITE:
+                        rookf = self.ini_rooks[0][1]
+                        rookt = F1
+                    else:
+                        rookf = self.ini_rooks[1][1]
+                        rookt = F8
+                    # don't move our rook yet
+                else:
+                    rookf = fcord + 3
+                    rookt = fcord + 1
+                    self._move (rookf, rookt, ROOK, self.color)
             self.hasCastled[self.color] = True
         
         if tpiece == EMPTY and fpiece != PAWN and \
@@ -343,23 +406,23 @@ class LBoard:
                     self.castling &= ~W_OO
                     
             if fpiece == ROOK:
-                if fcord == H1:
+                if fcord == self.ini_rooks[0][1]: #H1
                     if self.castling & W_OO:
                         self.hash ^= W_OOHash
                         self.castling &= ~W_OO
                     
-                elif fcord == A1:
+                elif fcord == self.ini_rooks[0][0]: #A1
                     if self.castling & W_OOO:
                         self.hash ^= W_OOOHash
                         self.castling &= ~W_OOO
             
             if tpiece == ROOK:
-                if tcord == H8:
+                if tcord == self.ini_rooks[1][1]: #H8
                     if self.castling & B_OO:
                         self.hash ^= B_OOHash
                         self.castling &= ~B_OO
             
-                elif tcord == A8:
+                elif tcord == self.ini_rooks[1][0]: #A8
                     if self.castling & B_OOO:
                         self.hash ^= B_OOOHash
                         self.castling &= ~B_OOO
@@ -374,29 +437,44 @@ class LBoard:
                     self.castling &= ~B_OO
             
             if fpiece == ROOK:
-                if fcord == H8:
+                if fcord == self.ini_rooks[1][1]: #H8
                     if self.castling & B_OO:
                         self.hash ^= B_OOHash
                         self.castling &= ~B_OO
             
-                elif fcord == A8:
+                elif fcord == self.ini_rooks[1][0]: #A8
                     if self.castling & B_OOO:
                         self.hash ^= B_OOOHash
                         self.castling &= ~B_OOO
             
             if tpiece == ROOK:
-                if tcord == H1:
+                if tcord == self.ini_rooks[0][1]: #H1
                     if self.castling & W_OO:
                         self.hash ^= W_OOHash
                         self.castling &= ~W_OO
                     
-                elif tcord == A1:
+                elif tcord == self.ini_rooks[0][0]: #A1
                     if self.castling & W_OOO:
                         self.hash ^= W_OOOHash
                         self.castling &= ~W_OOO
         
         if not flag in PROMOTIONS:
-            self._move(fcord, tcord, fpiece, self.color)
+            if self.variant == FISCHERRANDOMCHESS:
+                if flag in (KING_CASTLE, QUEEN_CASTLE):
+                    if tpiece == EMPTY:
+                        self._move(fcord, tcord, KING, self.color)
+                        self._move(rookf, rookt, ROOK, self.color)
+                    else:
+                        self._removePiece(rookf, ROOK, self.color)
+                        if flag == KING_CASTLE:
+                            self._move(fcord, rookt+1, KING, self.color)
+                        else:
+                            self._move(fcord, rookt-1, KING, self.color)
+                        self._addPiece(rookt, ROOK, self.color)
+                else:
+                    self._move(fcord, tcord, fpiece, self.color)
+            else:
+                self._move(fcord, tcord, fpiece, self.color)
         
         self.setColor(opcolor)
         self.updateBoard ()
@@ -404,7 +482,6 @@ class LBoard:
         return move # Move is returned with the captured piece flag set
     
     def popMove (self):
-        
         # Note that we remove the last made move, which was not made by boards
         # current color, but by its opponent
         color = 1 - self.color
@@ -420,15 +497,66 @@ class LBoard:
         
         tpiece = self.arBoard[tcord]
         
-        self._removePiece (tcord, tpiece, color)
+        if self.variant == FISCHERRANDOMCHESS:
+            if flag in (KING_CASTLE, QUEEN_CASTLE):
+                if color == WHITE:
+                    if flag == QUEEN_CASTLE:
+                        rookf = self.ini_rooks[0][0]
+                        rookt = D1
+                    else:
+                        rookf = self.ini_rooks[0][1]
+                        rookt = F1
+                else:
+                    if flag == QUEEN_CASTLE:
+                        rookf = self.ini_rooks[1][0]
+                        rookt = D8
+                    else:
+                        rookf = self.ini_rooks[1][1]
+                        rookt = F8
+                if cpiece == EMPTY:
+                    self._removePiece (tcord, KING, color)
+                else:
+                    if flag == KING_CASTLE:
+                        self._removePiece (rookt+1, KING, color)
+                    else:
+                        self._removePiece (rookt-1, KING, color)
+            else:
+                self._removePiece (tcord, tpiece, color)
+        else:
+            self._removePiece (tcord, tpiece, color)
+
+        # Put back rook moved by castling
+        if flag in (KING_CASTLE, QUEEN_CASTLE):
+            if self.variant == FISCHERRANDOMCHESS:
+                self._move (rookt, rookf, ROOK, color)
+            else:
+                if flag == QUEEN_CASTLE:
+                    rookf = fcord - 4
+                    rookt = fcord - 1
+                else:
+                    rookf = fcord + 3
+                    rookt = fcord + 1
+                self._move (rookt, rookf, ROOK, color)
+            self.hasCastled[color] = False
         
         # Put back captured piece
         if cpiece != EMPTY:
-            self._addPiece (tcord, cpiece, opcolor)
             if flag in PROMOTIONS:
+                self._addPiece (tcord, cpiece, opcolor)
                 self._addPiece (fcord, PAWN, color)
             else:
-                self._addPiece (fcord, tpiece, color)
+                if self.variant == FISCHERRANDOMCHESS:
+                    if flag in (KING_CASTLE, QUEEN_CASTLE):
+                        if flag == KING_CASTLE:
+                            self._addPiece (fcord, KING, color)
+                        else:
+                            self._addPiece (fcord, KING, color)
+                    else:
+                        self._addPiece (tcord, cpiece, opcolor)
+                        self._addPiece (fcord, tpiece, color)
+                else:
+                    self._addPiece (tcord, cpiece, opcolor)
+                    self._addPiece (fcord, tpiece, color)
         
         # Put back piece captured by enpassant
         elif flag == ENPASSANT:
@@ -443,16 +571,6 @@ class LBoard:
         else:
             self._addPiece (fcord, tpiece, color)
         
-        # Pyt back rook moved by castling
-        if flag in (KING_CASTLE, QUEEN_CASTLE):
-            if flag == QUEEN_CASTLE:
-                rookf = fcord - 4
-                rookt = fcord - 1
-            else:
-                rookf = fcord + 3
-                rookt = fcord + 1
-            self.hasCastled[color] = False
-            self._move (rookt, rookf, ROOK, color)
         
         self.setColor(color)
         self.updateBoard ()
@@ -472,14 +590,24 @@ class LBoard:
             return "-"
         else:
             strs = []
-            if self.castling & W_OO:
-                strs.append("K")
-            if self.castling & W_OOO:
-                strs.append("Q")
-            if self.castling & B_OO:
-                strs.append("k")
-            if self.castling & B_OOO:
-                strs.append("q")
+            if self.variant == FISCHERRANDOMCHESS:
+                if self.castling & W_OO:
+                    strs.append(reprCord[self.ini_rooks[0][1]][0].upper())
+                if self.castling & W_OOO:
+                    strs.append(reprCord[self.ini_rooks[0][0]][0].upper())
+                if self.castling & B_OO:
+                    strs.append(reprCord[self.ini_rooks[1][1]][0])
+                if self.castling & B_OOO:
+                    strs.append(reprCord[self.ini_rooks[1][0]][0])
+            else:
+                if self.castling & W_OO:
+                    strs.append("K")
+                if self.castling & W_OOO:
+                    strs.append("Q")
+                if self.castling & B_OO:
+                    strs.append("k")
+                if self.castling & B_OOO:
+                    strs.append("q")
             return "".join(strs)
     
     def __repr__ (self):
@@ -501,7 +629,7 @@ class LBoard:
             b += "\n"
         return b
     
-    def asFen (self):
+    def asFen (self, useXFen=False):
         fenstr = []
         
         rows = [self.arBoard[i:i+8] for i in range(0,64,8)][::-1]
@@ -535,7 +663,17 @@ class LBoard:
         if not self.enpassant:
             fenstr.append("-")
         else:
-            fenstr.append(reprCord[self.enpassant])
+            fwdPawns = self.boards[self.color][PAWN]
+            if self.color == WHITE:
+                fwdPawns >>= 8
+            else:
+                fwdPawns <<= 8
+            pawnTargets  = (fwdPawns & ~fileBits[0]) << 1;
+            pawnTargets |= (fwdPawns & ~fileBits[7]) >> 1;
+            if useXFen and not pawnTargets & bitPosArray[self.enpassant]:
+                fenstr.append("-")
+            else:
+                fenstr.append(reprCord[self.enpassant])
         fenstr.append(" ")
         
         fenstr.append(str(self.fifty))
@@ -545,3 +683,32 @@ class LBoard:
         fenstr.append(str(fullmove))
         
         return "".join(fenstr)
+    
+    def clone (self):
+        copy = LBoard(self.variant)
+        copy.blocker = self.blocker
+        
+        copy.friends = self.friends[:]
+        copy.kings = self.kings[:]
+        copy.boards = [self.boards[WHITE][:], self.boards[BLACK][:]]
+        
+        copy.enpassant = self.enpassant
+        copy.color = self.color
+        copy.castling = self.castling
+        copy.hasCastled = self.hasCastled[:]
+        copy.fifty = self.fifty
+        
+        copy.checked = self.checked
+        copy.opchecked = self.opchecked
+        
+        copy.arBoard = self.arBoard[:]
+        
+        copy.hash = self.hash
+        copy.pawnhash = self.pawnhash
+        
+        # We don't need to deepcopy the tupples, as they are imutable
+        copy.history = self.history[:]
+        
+        copy.ini_kings = self.ini_kings[:]
+        copy.ini_rooks = [self.ini_rooks[0][:], self.ini_rooks[1][:]]
+        return copy
