@@ -3,7 +3,6 @@
 import re
 
 from ldata import *
-from LBoard import LBoard
 from validator import validateMove
 from pychess.Utils.const import *
 
@@ -57,17 +56,20 @@ def parseAny (board, algnot):
     return parseFAN (board, algnot)
 
 def determineAlgebraicNotation (algnot):
-    
     upnot = algnot.upper()
-    
     if upnot in ("O-O", "O-O-O", "0-0", "0-0-0"):
         return SAN
     
+    # Test for e2-e4
     if "-" in algnot:
         return LAN
     
-    if (len(algnot) == 4 or (len(algnot) == 5 and upnot[4] in reprSign)) and \
-            algnot[:2] in cordDic and algnot[2:4] in cordDic:
+    # Test for b4xc5
+    if "x" in algnot and algnot.split('x')[0] in cordDic:
+        return LAN
+    
+    # Test for e2e4 or a7a8q or a7a8=q
+    if algnot[:2] in cordDic and algnot[2:4] in cordDic:
         return AN
     
     if algnot[0] in FAN_PIECES[WHITE] or algnot[0] in FAN_PIECES[BLACK]:
@@ -80,57 +82,47 @@ def determineAlgebraicNotation (algnot):
 ################################################################################
 
 def listToSan (board, moves):
+    # Work on a copy to ensure we don't break things
+    board = board.clone()
     sanmoves = []
-    
-    board.lock.acquire()
-    try:
-        for move in moves:
-            san = toSAN (board, move)
-            sanmoves.append(san)
-            board.applyMove(move)
-            
-        for move in moves:
-            board.popMove()
-    finally:
-        board.lock.release()
-    
+    for move in moves:
+        san = toSAN (board, move)
+        sanmoves.append(san)
+        board.applyMove(move)
     return sanmoves
 
 ################################################################################
 # listToMoves                                                                  #
 ################################################################################
 
-def listToMoves (board, movstrs, type=None, testvalidate=False):
+def listToMoves (board, movstrs, type=None, testvalidate=False, ignoreErrors=False):
+    # Work on a copy to ensure we don't break things
+    board = board.clone()
     moves = []
-    
-    board.lock.acquire()
-    try:
-        for mstr in movstrs:
-            try:
-                if type == None:
-                    move = parseAny (board, mstr)
-                elif type == SAN:
-                    move = parseSAN (board, mstr)
-                elif type == AN:
-                    move = parseAN (board, mstr)
-                elif type == LAN:
-                    move = parseLAN (board, mstr)
-            except ParsingError:
-                # We expect a ParsingError to be raised when parsing "old" lines
-                # from analyzing engines, which haven't yet noticed their new tasks
+
+    for mstr in movstrs:
+        try:
+            if type == None:
+                move = parseAny (board, mstr)
+            elif type == SAN:
+                move = parseSAN (board, mstr)
+            elif type == AN:
+                move = parseAN (board, mstr)
+            elif type == LAN:
+                move = parseLAN (board, mstr)
+        except ParsingError:
+            if ignoreErrors:
                 break
-            
-            if testvalidate:
-                if not validateMove (board, move):
-                    break
-            
-            moves.append(move)
-            board.applyMove(move)
-            
-        for move in moves:
-            board.popMove()
-    finally:
-        board.lock.release()
+            raise
+        
+        if testvalidate:
+            if not validateMove (board, move):
+                if not ignoreErrors:
+                    raise ParsingError, (mstr, 'Validation', board.asFen())
+                break 
+        
+        moves.append(move)
+        board.applyMove(move)
     
     return moves
 
@@ -173,21 +165,18 @@ def toSAN (board, move, localRepr=False):
         xs = []
         ys = []
         
-        board.lock.acquire()
-        try:
-            for altmove in genAllMoves(board):
-                mfcord = FCORD(altmove)
-                if board.arBoard[mfcord] == fpiece and \
-                        mfcord != fcord and \
-                        TCORD(altmove) == tcord:
-                    board.applyMove(altmove)
-                    if not board.opIsChecked():
-                        xs.append(FILE(mfcord))
-                        ys.append(RANK(mfcord))
-                    board.popMove()
-        finally:
-            board.lock.release()
-        
+        board_clone = board.clone()
+        for altmove in genAllMoves(board_clone):
+            mfcord = FCORD(altmove)
+            if board_clone.arBoard[mfcord] == fpiece and \
+                    mfcord != fcord and \
+                    TCORD(altmove) == tcord:
+                board_clone.applyMove(altmove)
+                if not board_clone.opIsChecked():
+                    xs.append(FILE(mfcord))
+                    ys.append(RANK(mfcord))
+                board_clone.popMove()
+
         x = FILE(fcord)
         y = RANK(fcord)
         
@@ -216,24 +205,19 @@ def toSAN (board, move, localRepr=False):
             notat += "="+localReprSign[PROMOTE_PIECE(flag)]
         else:
             notat += "="+reprSign[PROMOTE_PIECE(flag)]
-
-    board.lock.acquire()
-    try:
-        board.applyMove(move)
-        if board.isChecked():
-            for altmove in genAllMoves (board):
-                board.applyMove(altmove)
-                if board.opIsChecked():
-                    board.popMove()
-                    continue
-                board.popMove()
-                notat += "+"
-                break
-            else:
-                notat += "#"
-        board.popMove()
-    finally:
-        board.lock.release()
+    
+    board_clone = board.clone()
+    board_clone.applyMove(move)
+    if board_clone.isChecked():
+        for altmove in genAllMoves (board_clone):
+            board_clone.applyMove(altmove)
+            if board_clone.opIsChecked():
+                board_clone.popMove()
+                continue
+            notat += "+"
+            break
+        else:
+            notat += "#"
     
     return notat
 
@@ -243,14 +227,12 @@ def toSAN (board, move, localRepr=False):
 
 def parseSAN (board, san):
     """ Parse a Short/Abbreviated Algebraic Notation string """
-    
-    if len(san) < 2:
-        if not san:
-            raise ParsingError, (san, _("the move is an empty string"), board.asFen())
+    if not san:
+        raise ParsingError, (san, _("the move is an empty string"), board.asFen())
+    elif len(san) < 2:
         raise ParsingError, (san, _("the move is too short"), board.asFen())
     
     notat = san
-    color = board.color
     
     if notat[-1] in ("+", "#"):
         notat = notat[:-1]
@@ -270,22 +252,34 @@ def parseSAN (board, san):
     
     notat = notat.replace("0","O").replace("o","O")
     if notat.startswith("O-O"):
-        if color == WHITE:
-            fcord = E1
+        if board.color == WHITE:
+            fcord = board.ini_kings[0] #E1
             if notat == "O-O":
                 flag = KING_CASTLE
-                tcord = G1
+                if board.variant == FISCHERRANDOMCHESS:
+                    tcord = board.ini_rooks[0][1]
+                else:
+                    tcord = G1
             else:
                 flag = QUEEN_CASTLE
-                tcord = C1
+                if board.variant == FISCHERRANDOMCHESS:
+                    tcord = board.ini_rooks[0][0]
+                else:
+                    tcord = C1
         else:
-            fcord = E8
+            fcord = board.ini_kings[1] #E8
             if notat == "O-O":
                 flag = KING_CASTLE
-                tcord = G8
+                if board.variant == FISCHERRANDOMCHESS:
+                    tcord = board.ini_rooks[1][1]
+                else:
+                    tcord = G8
             else:
                 flag = QUEEN_CASTLE
-                tcord = C8
+                if board.variant == FISCHERRANDOMCHESS:
+                    tcord = board.ini_rooks[1][0]
+                else:
+                    tcord = C8
         
         return newMove (fcord, tcord, flag)
     
@@ -321,8 +315,8 @@ def parseSAN (board, san):
     if notat and notat[0] in reprRank:
         frank = int(notat[0])-1
         notat = notat[1:]
-    if notat and notat[0] and notat in reprFile:
-        ffile = ord(notat) - ord("a")
+    if notat and notat[0] in reprFile:
+        ffile = ord(notat[0]) - ord("a")
         notat = notat[1:]
     if notat and notat[0] in reprRank:
         frank = int(notat[0])-1
@@ -331,8 +325,7 @@ def parseSAN (board, san):
     # We find all pieces who could have done it. (If san was legal, there should
     # never be more than one)
     from lmovegen import genAllMoves
-    moves = [m for m in genAllMoves(board)]
-    for move in moves:
+    for move in genAllMoves(board):
         if TCORD(move) != tcord:
             continue
         f = FCORD(move)
@@ -345,15 +338,10 @@ def parseSAN (board, san):
         if flag in PROMOTIONS and FLAG(move) != flag:
             continue
         
-        board.lock.acquire()
-        try:
-            board.applyMove(move)
-            if board.opIsChecked():
-                board.popMove()
-                continue
-            board.popMove()
-        finally:
-            board.lock.release()
+        board_clone = board.clone()
+        board_clone.applyMove(move)
+        if board_clone.opIsChecked():
+            continue
         
         return move
     
@@ -417,13 +405,25 @@ def parseLAN (board, lan):
 # toAN                                                                         #
 ################################################################################
 
-def toAN (board, move):
+def toAN (board, move, short=False):
     """ Returns a Algebraic Notation string of a move
-        board should be prior to the move """
-    
+        board should be prior to the move
+        
+        short -- returns the short variant, e.g. f7f8q rather than f7f8=Q
+    """
     s = reprCord[FCORD(move)] + reprCord[TCORD(move)]
+    if board.variant == FISCHERRANDOMCHESS:
+        flag = move >> 12
+        if flag == KING_CASTLE:
+            return "O-O"
+        elif flag == QUEEN_CASTLE:
+            return "O-O-O"
+    
     if FLAG(move) in PROMOTIONS:
-        s += reprSign[PROMOTE_PIECE(FLAG(move))]
+        if short:
+            s += reprSign[PROMOTE_PIECE(FLAG(move))].lower()
+        else:
+            s += "=" + reprSign[PROMOTE_PIECE(FLAG(move))]
     return s
 
 ################################################################################
@@ -432,8 +432,9 @@ def toAN (board, move):
 
 def parseAN (board, an):
     """ Parse an Algebraic Notation string """
-    if not 4 <= len(an) <= 5:
-        raise ParsingError, (an, "the move must be 4 or 5 chars long", board.asFen())
+
+    if not 4 <= len(an) <= 6:
+        raise ParsingError, (an, "the move must be 4 or 6 chars long", board.asFen())
     
     try:
         fcord = cordDic[an[:2]]
@@ -441,17 +442,33 @@ def parseAN (board, an):
     except KeyError, e:
         raise ParsingError, (an, "the cord (%s) is incorrect" % e.args[0], board.asFen())
     
+    flag = NORMAL_MOVE
     if len(an) == 5:
+        #The a7a8q variant
         flag = chr2Sign[an[4].lower()] + 2
-    elif board.arBoard[fcord] == KING and fcord - tcord == 2:
-        flag = QUEEN_CASTLE
-    elif board.arBoard[fcord] == KING and fcord - tcord == -2:
-        flag = KING_CASTLE
+    elif len(an) == 6:
+        #The a7a8=q variant
+        flag = chr2Sign[an[5].lower()] + 2
+    elif board.arBoard[fcord] == KING:
+        if board.variant == FISCHERRANDOMCHESS and board.arBoard[tcord] == ROOK:
+            color = board.color
+            friends = board.friends[color]
+            if bitPosArray[tcord] & friends:
+                if board.ini_rooks[color][0] == tcord:
+                    flag = QUEEN_CASTLE
+                else:
+                    flag = KING_CASTLE
+                
+        elif fcord - tcord == 2:
+            flag = QUEEN_CASTLE
+        elif fcord - tcord == -2:
+            flag = KING_CASTLE
+        else:
+            flag = NORMAL_MOVE
     elif board.arBoard[fcord] == PAWN and board.arBoard[tcord] == EMPTY and \
             FILE(fcord) != FILE(tcord) and RANK(fcord) != RANK(tcord):
         flag = ENPASSANT
-    else: flag = NORMAL_MOVE
-    
+
     return newMove (fcord, tcord, flag)
 
 ################################################################################
@@ -489,48 +506,10 @@ def toFAN (board, move):
     
     lan = toSAN (board, move)
     
-    if not lan[0] in ("K", "Q", "R", "B", "N", "O"):
-        
-        # We generally don't want notations like Pexd4, as the from file - in
-        # this case 'e' could be cut out. However in some cases where two pawns
-        # can attack the same cord, we'll still need it.
-        i = lan.find("x")
-        if i >= 0:
-            fcord = FCORD(move)
-            tcord = TCORD(move)
-            fileNecessary = False
-            
-            from lmovegen import genAllMoves
-            board.lock.acquire()
-            try:
-                for altmove in genAllMoves(board):
-                    mfcord = FCORD(altmove)
-                    if board.arBoard[mfcord] == PAWN and \
-                            mfcord != fcord and \
-                            TCORD(altmove) == tcord:
-                        board.applyMove(altmove)
-                        if not board.opIsChecked():
-                            fileNecessary = True
-                        board.popMove()
-                        # If we found a pawn, that is not us, which can move to our
-                        # tcord, there is no point in looking further, as we can
-                        # never have more than two pawn pointing at the same cord
-                        break
-            finally:
-                board.lock.release()
-            
-            if not fileNecessary:
-                lan = lan[i:]
-        
-        # We add the pawn sign by appending instead of replacing, as SAN
-        # notation has no equal
-        if board.color == WHITE:
-            lan = FAN_PIECES[WHITE][PAWN] + lan
-        else: lan = FAN_PIECES[BLACK][PAWN] + lan
-    
     if board.color == WHITE:
         lan = san2WhiteFanRegex.sub(san2WhiteFanFunc, lan)
-    else: lan = san2BlackFanRegex.sub(san2BlackFanFunc, lan)
+    else:
+        lan = san2BlackFanRegex.sub(san2BlackFanFunc, lan)
     
     return lan
 
@@ -568,19 +547,16 @@ def parseFAN (board, fan):
             tcord = cordDic[tocord]
             
             from lmovegen import genAllMoves
-            board.lock.acquire()
-            try:
-                for altmove in genAllMoves(board):
-                    if board.arBoard[FCORD(altmove)] == PAWN and \
-                            TCORD(altmove) == tcord:
-                        board.applyMove(altmove)
-                        if not board.opIsChecked():
-                            san = reprFile(mfcord) + san
-                        board.popMove()
-                        # We know there is only one pawn which can move to tcord, so
-                        # we stop work here
-                        break
-            finally:
-                board.lock.release()
+            board_clone = board.clone()
+            for altmove in genAllMoves(board_clone):
+                if board_clone.arBoard[FCORD(altmove)] == PAWN and \
+                        TCORD(altmove) == tcord:
+                    board_clone.applyMove(altmove)
+                    if not board_clone.opIsChecked():
+                        san = reprFile(mfcord) + san
+                    board_clone.popMove()
+                    # We know there is only one pawn which can move to tcord, so
+                    # we stop work here
+                    break
     
     return parseSAN (board, san)
