@@ -13,10 +13,12 @@ from pychess.System import glock, conf, gstreamer
 from pychess.System.glock import glock_connect, glock_connect_after
 from pychess.System.repeat import repeat, repeat_sleep
 from pychess.gfx.Pieces import drawPiece
+from pychess.Utils.Piece import Piece
 from pychess.Utils.Cord import Cord
 from pychess.Utils.Move import Move
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.const import *
+from pychess.Variants.fischerandom import FischerRandomChess
 import preferencesDialog
 
 def intersects (r0, r1):
@@ -75,12 +77,14 @@ def matrixAround (rotatedMatrix, anchorX, anchorY):
     invmatrix = cairo.Matrix(co, -si, si, co, axco-aysi, ayco+axsi)
     return matrix, invmatrix
 
-ANIMATION_TIME = .5
+ANIMATION_TIME = 0.5
 
 # If this is true, the board is scaled so that everything fits inside the window
 # even if the board is rotated 45 degrees
 SCALE_ROTATED_BOARD = False
 
+CORD_PADDING = 1.5
+
 class BoardView (gtk.DrawingArea):
     
     __gsignals__ = {
@@ -94,15 +98,18 @@ class BoardView (gtk.DrawingArea):
             gamemodel = GameModel()
         self.model = gamemodel
         glock_connect(self.model, "game_started", self.game_started)
+        glock_connect_after(self.model, "game_started", self.game_started_after)
         glock_connect_after(self.model, "game_changed", self.game_changed)
         glock_connect_after(self.model, "moves_undoing", self.moves_undoing)
         glock_connect_after(self.model, "game_loading", self.game_loading)
         glock_connect_after(self.model, "game_loaded", self.game_loaded)
         glock_connect_after(self.model, "game_ended", self.game_ended)
+        
         self.connect("expose_event", self.expose)
         self.connect_after("realize", self.on_realized)
         conf.notify_add("showCords", self.on_show_cords)
-        self.set_size_request(300,300)
+        conf.notify_add("faceToFace", self.on_face_to_face)
+        self.set_size_request(350,350)
         
         self.animationStart = time()
         self.lastShown = None
@@ -114,12 +121,14 @@ class BoardView (gtk.DrawingArea):
         self.square = 0, 0, 8, 1 # An object global variable with the current
                                  # board size
         self.pad = 0.13 # Padding applied only when setcords is active
+        
         self._selected = None
         self._hover = None
         self._active = None
         self._redarrow = None
         self._greenarrow = None
         self._bluearrow = None
+        
         self._shown = self.model.ply
         self._showCords = False
         self.showCords = conf.get("showCords", False)
@@ -133,11 +142,18 @@ class BoardView (gtk.DrawingArea):
         self.drawcount = 0
         self.drawtime = 0
         
-        self.animationLock = Lock()
+        self.gotStarted = False
+        self.animationLock = RLock()
         self.rotationLock = Lock()
+        
+        self.draggedPiece = None  # a piece being dragged by the user
+    
+    def game_started_after (self, model):
+        self.emit("shown_changed", self.shown)
     
     def game_started (self, model):
         if conf.get("noAnimation", False):
+            self.gotStarted = True
             self.redraw_canvas()
         else:
             if model.moves:
@@ -150,14 +166,14 @@ class BoardView (gtk.DrawingArea):
                             piece.opacity = 0
             finally:
                 self.animationLock.release()
+            self.gotStarted = True
             self.startAnimation()
     
     def game_changed (self, model):
         # Play sounds
-        if self.model.players and conf.get("useSounds", False) and \
-                self.model.status != WAITING_TO_START:
+        if self.model.players and self.model.status != WAITING_TO_START:
             move = model.moves[-1]
-            if move.flag == ENPASSANT or model.boards[-2][move.cord1] != None:
+            if move.is_capture(model.boards[-2]):
                 sound = "aPlayerCaptures"
             else: sound = "aPlayerMoves"
             
@@ -184,18 +200,16 @@ class BoardView (gtk.DrawingArea):
     def moves_undoing (self, model, moves):
         self.shown = model.ply-moves
     
-    def game_loading (self, model):
+    def game_loading (self, model, uri):
         self.autoUpdateShown = False
-    
     def game_loaded (self, model, uri):
         self.autoUpdateShown = True
         self._shown = model.ply
-        self.emit("shown_changed", self.shown)
     
     def game_ended (self, model, reason):
         self.redraw_canvas()
         
-        if self.model.players and conf.get("useSounds", False):
+        if self.model.players:
             sound = False
             
             if model.status == DRAW:
@@ -226,10 +240,30 @@ class BoardView (gtk.DrawingArea):
     def on_show_cords (self, *args):
         self.showCords = conf.get("showCords", False)
     
+    def on_face_to_face (self, *args):
+        self.redraw_canvas()
+    
     ###############################
     #          Animation          #
     ###############################
     
+    def paintBoxAround(self, move):
+        paintBox = self.cord2RectRelative(move.cord0)
+        paintBox = join(paintBox, self.cord2RectRelative(move.cord1))
+        if move.flag in (KING_CASTLE, QUEEN_CASTLE):
+            y = move.cord0.cy
+            color = (y == 1)
+            rsqs = self.model.boards[-1].board.ini_rooks[color]
+            if move.flag == KING_CASTLE:
+                paintBox = join(paintBox, self.cord2RectRelative(Cord(rsqs[1])))
+                paintBox = join(paintBox, self.cord2RectRelative(Cord("f" + y)))
+                paintBox = join(paintBox, self.cord2RectRelative(Cord("g" + y)))
+            else:
+                paintBox = join(paintBox, self.cord2RectRelative(Cord(rsqs[0])))
+                paintBox = join(paintBox, self.cord2RectRelative(Cord("c" + y)))
+                paintBox = join(paintBox, self.cord2RectRelative(Cord("d" + y)))
+        return paintBox
+
     def _get_shown(self):
         return self._shown
     
@@ -253,50 +287,44 @@ class BoardView (gtk.DrawingArea):
             self.redraw_canvas()
             return
         
-        # TODO: This should be faster and be able to support fade at promotion,
-        # if it is rewritten to be based on <Move>s instead of boards. Perhaps
-        # it would share code with the Board .move method
         
         step = shown > self.shown and 1 or -1
         
         self.animationLock.acquire()
         try:
             deadset = set()
-            for i in range(self.shown, shown, step):
-                board0 = self.model.getBoardAtPly(i)
-                board1 = self.model.getBoardAtPly(i+step)
+            for i in xrange(self.shown, shown, step):
+                board = self.model.getBoardAtPly(i)
+                board1 = self.model.getBoardAtPly(i + step)
+                if step == 1:
+                    move = self.model.getMoveAtPly(i)
+                    moved, new, dead = board.simulateMove(board1, move)
+                else:
+                    move = self.model.getMoveAtPly(i-1)
+                    moved, new, dead = board.simulateUnmove(board1, move)
                 
-                for y, row in enumerate(board0.data):
-                    for x, piece in enumerate(row):
-                        if not piece: continue
-                        
-                        if step < 0 and piece.opacity < 1:
-                            # If piece is fading in, it should not move
-                            continue
-                            
-                        if step > 0 and piece in deadset:
-                            # No need for more testing, if piece is dead
-                            continue
-                            
-                        if piece != board1.data[y][x]:
-                            
-                            dir = board0.color == WHITE and 1 or -1
-                            if step > 0 and board1.data[y][x] != None or \
-                                    0 < y < 7 and board0.enpassant == Cord(x,y+dir)\
-                                    and board1[board0.enpassant] != None:
-                                    
-                                # A piece is dying
-                                deadset.add(piece)
-                                
-                                # If dead pieces as a location, they jump a little
-                                # When they are waken to life
-                                piece.x = None
-                                piece.y = None
-                                
-                            elif piece.x == None:
-                                # It has moved
-                                piece.x = x
-                                piece.y = y
+                # We need to ensure, that the piece coordinate is saved in the
+                # piece
+                for piece, cord0 in moved:
+                    # Test if the piece already has a realcoord (has been dragged)
+                    if piece.x == None:
+                        # We don't want newly restored pieces to flew from their
+                        # deadspot to their old position, as it doesn't work
+                        # vice versa  
+                        if piece.opacity == 1:
+                            piece.x = cord0.x
+                            piece.y = cord0.y
+                
+                for piece in dead:
+                    deadset.add(piece)
+                    # Reset the location of the piece to avoid a small visual
+                    # jump, when it is at some other time waken to life.
+                    piece.x = None
+                    piece.y = None
+                
+                for piece in new:
+                    piece.opacity = 0
+        
         finally:
             self.animationLock.release()
         
@@ -311,14 +339,14 @@ class BoardView (gtk.DrawingArea):
         
         self.animationStart = time()
         if self.lastMove:
-            paintBox = self.cord2RectRelative(self.lastMove.cord0)
-            paintBox = join(paintBox, self.cord2RectRelative(self.lastMove.cord1))
+            paintBox = self.paintBoxAround(self.lastMove)
             self.lastMove = None
             self.redraw_canvas(rect(paintBox))
         if self.shown > self.model.lowply:
             self.lastMove = self.model.getMoveAtPly(self.shown-1)
         else:
             self.lastMove = None
+       
         self.runAnimation(redrawMisc=True)
         repeat(self.runAnimation)
         
@@ -333,29 +361,32 @@ class BoardView (gtk.DrawingArea):
         try:
             paintBox = None
             
-            mod = min(1.0, (time()-self.animationStart)/ANIMATION_TIME)
+            mod = min(1, (time()-self.animationStart)/ANIMATION_TIME)
             board = self.model.getBoardAtPly(self.shown)
             
             for y, row in enumerate(board.data):
                 for x, piece in enumerate(row):
                     if not piece: continue
+                    if piece == self.draggedPiece: continue
                     
                     if piece.x != None:
                         if not conf.get("noAnimation", False):
-                            newx = piece.x + (x-piece.x)*mod
-                            newy = piece.y + (y-piece.y)*mod
+                            if piece.piece == KNIGHT:
+                                #print mod, x, piece.x
+                                newx = piece.x + (x-piece.x)*mod**(1.5)
+                                newy = piece.y + (y-piece.y)*mod
+                            else:
+                                newx = piece.x + (x-piece.x)*mod
+                                newy = piece.y + (y-piece.y)*mod
                         else:
                             newx, newy = x, y
                         
-                        if not paintBox:
-                            paintBox = self.cord2RectRelative(piece.x, piece.y)
-                        else: paintBox = join(paintBox,
-                                self.cord2RectRelative(piece.x, piece.y))
+                        paintBox = join(paintBox, self.cord2RectRelative(piece.x, piece.y))
                         paintBox = join(paintBox, self.cord2RectRelative(newx, newy))
                         
                         if (newx <= x <= piece.x or newx >= x >= piece.x) and \
                            (newy <= y <= piece.y or newy >= y >= piece.y) or \
-                           abs(newx-x) < 0.01 and abs(newy-y) < 0.01:
+                           abs(newx-x) < 0.005 and abs(newy-y) < 0.005:
                             piece.x = None
                             piece.y = None
                         else:
@@ -379,7 +410,7 @@ class BoardView (gtk.DrawingArea):
                         else:
                             newOp = 1
                         
-                        if newOp >= 1 >= piece.opacity or abs(1-newOp) < 0.01:
+                        if newOp >= 1 >= piece.opacity or abs(1-newOp) < 0.005:
                             piece.opacity = 1
                         else: piece.opacity = newOp
             
@@ -393,7 +424,7 @@ class BoardView (gtk.DrawingArea):
                 else:
                     newOp = 0
                 
-                if newOp <= 0 <= piece.opacity or abs(0-newOp) < 0.01:
+                if newOp <= 0 <= piece.opacity or abs(0-newOp) < 0.005:
                     del self.deadlist[i]
                 else: piece.opacity = newOp
         
@@ -410,9 +441,7 @@ class BoardView (gtk.DrawingArea):
                     paintBox = join(paintBox, self.cord2RectRelative(arrow[1]))
             if self.lastMove:
                 paintBox = join(paintBox,
-                                self.cord2RectRelative(self.lastMove.cord0))
-                paintBox = join(paintBox,
-                                self.cord2RectRelative(self.lastMove.cord1))
+                                self.paintBoxAround(self.lastMove))
         
         if paintBox:
             self.redraw_canvas(rect(paintBox))
@@ -453,7 +482,9 @@ class BoardView (gtk.DrawingArea):
         else:
             self.drawcount += 1
             start = time()
+            self.animationLock.acquire()
             self.draw(context, event.area)
+            self.animationLock.release()
             self.drawtime += time() - start
             #if self.drawcount % 100 == 0:
             #    print "Average FPS: %0.3f - %d / %d" % \
@@ -469,7 +500,7 @@ class BoardView (gtk.DrawingArea):
     #        redraw_canvas        #
     ###############################
     
-    def redraw_canvas(self, r=None):
+    def redraw_canvas(self, r=None, queue=False):
         if self.window:
             glock.acquire()
             try:
@@ -478,8 +509,11 @@ class BoardView (gtk.DrawingArea):
                         alloc = self.get_allocation()
                         r = gtk.gdk.Rectangle(0, 0, alloc.width, alloc.height)
                     assert type(r[2]) == int
-                    self.window.invalidate_rect(r, True)
-                    self.window.process_updates(True)
+                    if queue:
+                        self.queue_draw_area(r.x, r.y, r.width, r.height)
+                    else:
+                        self.window.invalidate_rect(r, True)
+                        self.window.process_updates(True)
             finally:
                 glock.release()
     
@@ -510,9 +544,11 @@ class BoardView (gtk.DrawingArea):
         self.square = (xc, yc, square, s)
         
         self.drawBoard (context, r)
-        self.drawCords (context, r)
         
-        if self.model.status != WAITING_TO_START:
+        if min(alloc.width, alloc.height) > 32:
+            self.drawCords (context, r)
+        
+        if self.gotStarted:
             self.drawSpecial (context, r)
             self.drawEnpassant (context, r)
             self.drawArrows (context)
@@ -633,51 +669,75 @@ class BoardView (gtk.DrawingArea):
     #         drawPieces          #
     ###############################
     
-    def getCordMatrices (self, x, y):
+    def getCordMatrices (self, x, y, inv=False):
         xc, yc, square, s = self.square
         square_, rot_ = self.cordMatricesState
         if square != self.square or rot_ != self.rotation:
             self.cordMatrices = [None] * 64
             self.cordMatricesState = (self.square, self.rotation)
         c = x * 8 + y
-        matrices = self.cordMatrices[c]
-        if matrices == None:
+        if type(c) == int and self.cordMatrices[c]:
+            matrices = self.cordMatrices[c]
+        else:
             cx, cy = self.cord2Point(x,y)
             matrices = matrixAround(self.matrix, cx+s/2., cy+s/2.)
             matrices += (cx, cy)
-            self.cordMatrices[c] = matrices
+            if type(c) == int:
+                self.cordMatrices[c] = matrices
         return matrices
     
+    def __drawPiece(self, context, piece, x, y):
+        xc, yc, square, s = self.square
+        
+        if not conf.get("faceToFace", False):
+            matrix, invmatrix, cx, cy = self.getCordMatrices(x, y)
+        else:
+            cx, cy = self.cord2Point(x,y)
+            if piece.color == BLACK:
+                matrix, invmatrix = matrixAround((-1,0), cx+s/2., cy+s/2.)
+            else:
+                matrix = invmatrix = cairo.Matrix(1,0,0,1,0,0)
+        
+        context.transform(invmatrix)
+        drawPiece(  piece, context,
+                    cx+CORD_PADDING, cy+CORD_PADDING,
+                    s-CORD_PADDING*2)
+        context.transform(matrix)
+    
     def drawPieces(self, context, r):
         pieces = self.model.getBoardAtPly(self.shown)
         xc, yc, square, s = self.square
         
-        CORD_BORDER = 1.5
+        parseC = lambda c: (c.red/65535., c.green/65535., c.blue/65535.)
+        fgN = parseC(self.get_style().fg[gtk.STATE_NORMAL])
+        fgS = fgN
+        fgA = parseC(self.get_style().fg[gtk.STATE_ACTIVE])
+        fgP = parseC(self.get_style().fg[gtk.STATE_PRELIGHT])
+        
+        # As default we use normal foreground for selected cords, as it looks
+        # less confusing. However for some themes, the normal foreground is so
+        # similar to the selected background, that we have to use the selected
+        # foreground.
+        bgSl = parseC(self.get_style().bg[gtk.STATE_SELECTED])
+        bgSd = parseC(self.get_style().dark[gtk.STATE_SELECTED])
+        if min((fgN[0]-bgSl[0])**2+(fgN[1]-bgSl[1])**2+(fgN[2]-bgSl[2])**2,
+               (fgN[0]-bgSd[0])**2+(fgN[1]-bgSd[1])**2+(fgN[2]-bgSd[2])**2) < 0.2:
+            fgS = parseC(self.get_style().fg[gtk.STATE_SELECTED])
         
         # Draw dying pieces (Found in self.deadlist)
         for piece, x, y in self.deadlist:
-            matrix, invmatrix, cx, cy = self.getCordMatrices(x, y)
-            context.transform(invmatrix)
-            context.set_source_rgba(0,0,0,piece.opacity)
-            drawPiece(  piece, context,
-                        cx+CORD_BORDER, cy+CORD_BORDER,
-                        s-CORD_BORDER*2)
-            context.transform(matrix)
+            context.set_source_rgba(fgN[0],fgN[1],fgN[2],piece.opacity)
+            self.__drawPiece(context, piece, x, y)
         
         # Draw pieces reincarnating (With opacity < 1)
         for y, row in enumerate(pieces.data):
             for x, piece in enumerate(row):
                 if not piece or piece.opacity == 1:
                     continue
-                matrix, invmatrix, cx, cy = self.getCordMatrices(x, y)
-                context.transform(invmatrix)
-                context.set_source_rgba(0,0,0,piece.opacity)
-                drawPiece(  piece, context,
-                            cx+CORD_BORDER, cy+CORD_BORDER,
-                            s-CORD_BORDER*2)
-                context.transform(matrix)
-        
-        context.set_source_rgb(0,0,0)
+                if piece.x:
+                    x, y = piece.x, piece.y
+                context.set_source_rgba(fgN[0],fgN[1],fgN[2],piece.opacity)
+                self.__drawPiece(context, piece, x, y)
         
         # Draw standing pieces (Only those who intersect drawn area)
         for y, row in enumerate(pieces.data):
@@ -686,25 +746,25 @@ class BoardView (gtk.DrawingArea):
                     continue
                 if not intersects(rect(self.cord2RectRelative(x,y)), r):
                     continue
-                matrix, invmatrix, cx, cy = self.getCordMatrices(x, y)
-                context.transform(invmatrix)
-                drawPiece(  piece, context,
-                            cx+CORD_BORDER, cy+CORD_BORDER,
-                            s-CORD_BORDER*2)
-                context.transform(matrix)
+                
+                if Cord(x,y) == self.selected:
+                    context.set_source_rgb(*fgS)
+                elif Cord(x,y) == self.active:
+                    context.set_source_rgb(*fgA)
+                elif Cord(x,y) == self.hover:
+                    context.set_source_rgb(*fgP)
+                else: context.set_source_rgb(*fgN)
+                
+                self.__drawPiece(context, piece, x, y)
+        
+        context.set_source_rgb(*fgP)
         
         # Draw moving or dragged pieces (Those with piece.x and piece.y != None)
         for y, row in enumerate(pieces.data):
             for x, piece in enumerate(row):
                 if not piece or piece.x == None or piece.opacity < 1:
                     continue
-                cx, cy = self.cord2Point(piece.x, piece.y)
-                matrix, invmatrix = matrixAround(self.matrix, cx+s/2., cy+s/2.)
-                context.transform(invmatrix)
-                drawPiece(  piece, context,
-                            cx+CORD_BORDER, cy+CORD_BORDER,
-                            s-CORD_BORDER*2)
-                context.transform(matrix)
+                self.__drawPiece(context, piece, piece.x, piece.y)
     
     ###############################
     #         drawSpecial         #
@@ -739,9 +799,10 @@ class BoardView (gtk.DrawingArea):
     
     def drawLastMove (self, context, redrawn):
         if not self.lastMove: return
-        ply = self.shown-1
-        if ply < self.model.lowply: return
-        capture = self.model.getBoardAtPly(ply)[self.lastMove.cord1]
+        if self.shown <= self.model.lowply: return
+        show_board = self.model.getBoardAtPly(self.shown)
+        last_board = self.model.getBoardAtPly(self.shown - 1)
+        capture = self.lastMove.is_capture(last_board)
         
         wh = 0.27 # Width of marker
         p0 = 0.155 # Padding on last cord
@@ -762,58 +823,113 @@ class BoardView (gtk.DrawingArea):
         light_orange = (.961, .475, 0, 0.8)
         dark_orange  = (.808, .361, 0, 0.5)
         
-        
-        rel = self.cord2RectRelative(self.lastMove.cord0)
-        if intersects(rect(rel), redrawn):
-            r = self.cord2Rect(self.lastMove.cord0)
-            for m in ms:
-                context.move_to(
-                    r[0]+(d0[m[0]]+wh*m[0])*r[2],
-                    r[1]+(d0[m[1]]+wh*m[1])*r[2])
-                context.rel_line_to(
-                    0, -wh*r[2]*m[1])
-                context.rel_curve_to(
-                    0, wh*r[2]*m[1]/2.0,
-                    -wh*r[2]*m[0]/2.0, wh*r[2]*m[1],
-                    -wh*r[2]*m[0], wh*r[2]*m[1])
-                context.close_path()
-            
-            context.set_source_rgba(*light_yellow)
-            context.fill_preserve()
-            context.set_source_rgba(*dark_yellow)
-            context.stroke()
-            
-        rel = self.cord2RectRelative(self.lastMove.cord1)
-        if intersects(rect(rel), redrawn):
-            r = self.cord2Rect(self.lastMove.cord1)
-            
-            for m in ms:
-                context.move_to(
-                    r[0]+d1[m[0]]*r[2],
-                    r[1]+d1[m[1]]*r[2])
-                context.rel_line_to(
-                    wh*r[2]*m[0], 0)
-                context.rel_curve_to(
-                    -wh*r[2]*m[0]/2.0, 0,
-                    -wh*r[2]*m[0], wh*r[2]*m[1]/2.0,
-                    -wh*r[2]*m[0], wh*r[2]*m[1])
-                context.close_path()
-            
-            if capture:
-                context.set_source_rgba(*light_orange)
-                context.fill_preserve()
-                context.set_source_rgba(*dark_orange)
-                context.stroke()
+        if self.lastMove.flag in (KING_CASTLE, QUEEN_CASTLE):
+            ksq0 = last_board.board.kings[last_board.color]
+            ksq1 = show_board.board.kings[last_board.color]
+            if self.lastMove.flag == KING_CASTLE:
+                rsq0 = show_board.board.ini_rooks[last_board.color][1]
+                rsq1 = ksq1 - 1
             else:
+                rsq0 = show_board.board.ini_rooks[last_board.color][0]
+                rsq1 = ksq1 + 1
+            cord_pairs = [ [Cord(ksq0), Cord(ksq1)], [Cord(rsq0), Cord(rsq1)] ]
+        else:
+            cord_pairs = [ [self.lastMove.cord0, self.lastMove.cord1] ]
+
+        for [cord0, cord1] in cord_pairs:
+            rel = self.cord2RectRelative(cord0)
+            if intersects(rect(rel), redrawn):
+                r = self.cord2Rect(cord0)
+                for m in ms:
+                    context.move_to(
+                        r[0]+(d0[m[0]]+wh*m[0])*r[2],
+                        r[1]+(d0[m[1]]+wh*m[1])*r[2])
+                    context.rel_line_to(
+                        0, -wh*r[2]*m[1])
+                    context.rel_curve_to(
+                        0, wh*r[2]*m[1]/2.0,
+                        -wh*r[2]*m[0]/2.0, wh*r[2]*m[1],
+                        -wh*r[2]*m[0], wh*r[2]*m[1])
+                    context.close_path()
+                
                 context.set_source_rgba(*light_yellow)
                 context.fill_preserve()
                 context.set_source_rgba(*dark_yellow)
                 context.stroke()
+                
+            rel = self.cord2RectRelative(cord1)
+            if intersects(rect(rel), redrawn):
+                r = self.cord2Rect(cord1)
+                
+                for m in ms:
+                    context.move_to(
+                        r[0]+d1[m[0]]*r[2],
+                        r[1]+d1[m[1]]*r[2])
+                    context.rel_line_to(
+                        wh*r[2]*m[0], 0)
+                    context.rel_curve_to(
+                        -wh*r[2]*m[0]/2.0, 0,
+                        -wh*r[2]*m[0], wh*r[2]*m[1]/2.0,
+                        -wh*r[2]*m[0], wh*r[2]*m[1])
+                    context.close_path()
+                
+                if capture:
+                    context.set_source_rgba(*light_orange)
+                    context.fill_preserve()
+                    context.set_source_rgba(*dark_orange)
+                    context.stroke()
+                else:
+                    context.set_source_rgba(*light_yellow)
+                    context.fill_preserve()
+                    context.set_source_rgba(*dark_yellow)
+                    context.stroke()
     
     ###############################
     #         drawArrows          #
     ###############################
     
+    def __drawArrow (self, context, cords, aw, ahw, ahh, asw, fillc, strkc):
+        context.save()
+        
+        lvx = cords[1].x-cords[0].x
+        lvy = cords[0].y-cords[1].y
+        l = float((lvx**2+lvy**2)**.5)
+        vx = lvx/l
+        vy = lvy/l
+        v1x = -vy
+        v1y = vx
+        
+        r = self.cord2Rect(cords[0])
+        
+        px = r[0]+r[2]/2.0
+        py = r[1]+r[2]/2.0
+        ax = v1x*r[2]*aw/2
+        ay = v1y*r[2]*aw/2
+        context.move_to(px+ax, py+ay)
+        
+        p1x = px+(lvx-vx*ahh)*r[2]
+        p1y = py+(lvy-vy*ahh)*r[2]
+        context.line_to(p1x+ax, p1y+ay)
+        
+        lax = v1x*r[2]*ahw/2
+        lay = v1y*r[2]*ahw/2
+        context.line_to(p1x+lax, p1y+lay)
+        
+        context.line_to(px+lvx*r[2], py+lvy*r[2])
+        context.line_to(p1x-lax, p1y-lay)
+        context.line_to(p1x-ax, p1y-ay)
+        context.line_to(px-ax, py-ay)
+        context.close_path()
+        
+        context.set_source_rgba(*fillc)
+        context.fill_preserve()
+        context.set_line_join(gtk.gdk.JOIN_ROUND)
+        context.set_line_width(asw*r[2])
+        context.set_source_rgba(*strkc)
+        context.stroke()
+        
+        context.restore()
+    
     def drawArrows (self, context):
         # TODO: Only redraw when intersecting with the redrawn area
         
@@ -821,53 +937,21 @@ class BoardView (gtk.DrawingArea):
         ahw = 0.72 # Arrow head width
         ahh = 0.64 # Arrow head height
         asw = 0.08 # Arrow stroke width
-        def drawArrow (cords, fillc, strkc):
-            context.save()
-            
-            lvx = cords[1].x-cords[0].x
-            lvy = cords[0].y-cords[1].y
-            from math import sqrt
-            l = float(sqrt(lvx**2+lvy**2))
-            vx = lvx/l
-            vy = lvy/l
-            v1x = -vy
-            v1y = vx
-            r = self.cord2Rect(cords[0])
-            px = r[0]+r[2]/2.0
-            py = r[1]+r[2]/2.0
-            ax = v1x*r[2]*aw/2
-            ay = v1y*r[2]*aw/2
-            context.move_to(px+ax, py+ay)
-            p1x = px+(lvx-vx*ahh)*r[2]
-            p1y = py+(lvy-vy*ahh)*r[2]
-            context.line_to(p1x+ax, p1y+ay)
-            lax = v1x*r[2]*ahw/2
-            lay = v1y*r[2]*ahw/2
-            context.line_to(p1x+lax, p1y+lay)
-            context.line_to(px+lvx*r[2], py+lvy*r[2])
-            context.line_to(p1x-lax, p1y-lay)
-            context.line_to(p1x-ax, p1y-ay)
-            context.line_to(px-ax, py-ay)
-            context.close_path()
-            
-            context.set_source_rgba(*fillc)
-            context.fill_preserve()
-            context.set_line_join(gtk.gdk.JOIN_ROUND)
-            context.set_line_width(asw*r[2])
-            context.set_source_rgba(*strkc)
-            context.stroke()
-            context.restore()
         
         if self.bluearrow:
-            drawArrow(self.bluearrow, (.447,.624,.812,0.9), (.204,.396,.643,1))
+            self.__drawArrow(context, self.bluearrow, aw, ahw, ahh, asw,
+                             (.447,.624,.812,0.9), (.204,.396,.643,1))
         
         if self.shown != self.model.ply:
             return
         
         if self.greenarrow:
-            drawArrow(self.greenarrow, (.54,.886,.2,0.9), (.306,.604,.024,1))
+            self.__drawArrow(context, self.greenarrow, aw, ahw, ahh, asw,
+                             (.54,.886,.2,0.9), (.306,.604,.024,1))
+        
         if self.redarrow:
-            drawArrow(self.redarrow, (.937,.16,.16,0.9), (.643,0,0,1))
+            self.__drawArrow(context, self.redarrow, aw, ahw, ahh, asw,
+                             (.937,.16,.16,0.9), (.643,0,0,1))
     
     ###############################
     #        drawEnpassant        #
@@ -1096,9 +1180,9 @@ class BoardView (gtk.DrawingArea):
         x1, y1 = self.matrix.transform_point(cx+s, cy)
         x2, y2 = self.matrix.transform_point(cx, cy+s)
         x3, y3 = self.matrix.transform_point(cx+s, cy+s)
-        x = min((x0, x1, x2, x3))
-        y = min((y0, y1, y2, y3))
-        s = max((y0, y1, y2, y3)) - y
+        x = min(x0, x1, x2, x3)
+        y = min(y0, y1, y2, y3)
+        s = max(y0, y1, y2, y3) - y
         return (x, y, s)
     
     def isLight (self, cord):
